/**
 * This file is part of Carma Launcher.
 * 
 * Carma Launcher is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * Carma Launcher is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Carma Launcher. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Copyright 2011, Casper Kejlberg-Rasmussen.
 */
package masterserver;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 *
 * @author Casper Kejlberg-Rasmussen
 */
public class MasterServer{
    private int port;
    private int cleanupPort;
    private int cleanUpInterval;
    private Map<BigInteger, ServerInfo> serverlist;
    
    public MasterServer(int masterPort, int cleanupPort, int cleanupInterval){
        this.port = masterPort;
        this.cleanupPort = cleanupPort;
        this.cleanUpInterval = cleanupInterval;
        serverlist = new ConcurrentHashMap<BigInteger, ServerInfo>();
    }
    
    public void startServer(){
        try {
            ServerSocket ss = new ServerSocket(port);
            System.out.println("Server running ...");
            // Start the cleanup thead
            CleanUpServer cus = new CleanUpServer(serverlist, cleanupPort, cleanUpInterval);
            Thread tt = new Thread(cus);
            tt.start();
            while (true) {
                final Socket s = ss.accept();
                Runnable r = new Runnable() {
                    @Override
                    public void run() {
                        try {
                            ObjectInputStream ois = new ObjectInputStream(s.getInputStream());
                            ServerInfo serverInfo = (ServerInfo) ois.readObject();
                            serverInfo.setServerIP(s.getInetAddress());
                            switch (serverInfo.getType()) {
                                case RetrieveServerList: // Client wants a server list
                                    ObjectOutputStream oos = new ObjectOutputStream(s.getOutputStream());
                                    oos.writeObject(serverlist);
                                    oos.flush();
                                    oos.close();
                                    System.out.println("RetrieveServerList: " + serverInfo.getServerIP());
                                    System.out.println("Content: " + serverlist.keySet());
                                    break;
                                case PublishServer: // Client wants to publish a server
                                    serverlist.put(serverInfo.id(), serverInfo);
                                    System.out.println("PublishServer: " + serverInfo.getServerIP());
                                    System.out.println("Content: " + serverlist.keySet());
                                    break;
                                case RetractServer:
                                    serverlist.remove(serverInfo.id());
                                    System.out.println("RetractServer: " + serverInfo.getServerIP());
                                    System.out.println("Content: " + serverlist.keySet());
                                    break;
                                case JoinServer: // Client wants to register that he joins a server
                                    if (serverlist.containsKey(serverInfo.id())) {
                                        ServerInfo si = serverlist.get(serverInfo.id());
                                        si.getClients().add(s.getInetAddress());
                                    }
                                    System.out.println("JoinServer: " + serverInfo.getServerIP());
                                    System.out.println("Content: " + serverlist.keySet());
                                    break;
                                case LeaveServer:
                                    if (serverlist.containsKey(serverInfo.id())) {
                                        ServerInfo si = serverlist.get(serverInfo.id());
                                        si.getClients().remove(s.getInetAddress());
                                    }
                                    System.out.println("LeaveServer: " + serverInfo.getServerIP());
                                    System.out.println("Content: " + serverlist.keySet());
                                    break;
                            }
                            ois.close();
                            s.close();
                        } catch (ClassNotFoundException ex) {
                            System.out.println("Error communicating with a client\n" + ex.getMessage());
                        } catch (IOException ex) {
                            System.out.println("Error connecting to a client\n" + ex.getMessage());
                        }
                    }
                };
                Thread t = new Thread(r);
                t.start();
            }
        } catch(IOException ex){
            System.out.println("Error initializing server\n" + ex.getMessage());
        }
    }
}
